@@ -22,7 +22,7 @@ from werkzeug.serving import run_with_reloader
 from .namespace import Namespace
 from .test_client import SocketIOTestClient
 
-__version__ = '2.7.1'
+__version__ = '2.8.5'
 
 
 class _SocketIOMiddleware(socketio.Middleware):
@@ -36,6 +36,7 @@ class _SocketIOMiddleware(socketio.Middleware):
                                                   socketio_path)
 
     def __call__(self, environ, start_response):
+        environ = environ.copy()
         environ['flask.app'] = self.flask_app
         return super(_SocketIOMiddleware, self).__call__(environ,
                                                          start_response)
@@ -121,21 +122,29 @@ class SocketIO(object):
 
     def __init__(self, app=None, **kwargs):
         self.server = None
-        self.server_options = None
+        self.server_options = {}
         self.wsgi_server = None
         self.handlers = []
         self.namespace_handlers = []
         self.exception_handlers = {}
         self.default_exception_handler = None
-        if app is not None or len(kwargs) > 0:
+        # We can call init_app when:
+        # - we were given the Flask app instance (standard initialization)
+        # - we were not given the app, but we were given a message_queue
+        #   (standard initialization for auxiliary process)
+        # In all other cases we collect the arguments and assume the client
+        # will call init_app from an app factory function.
+        if app is not None or 'message_queue' in kwargs:
             self.init_app(app, **kwargs)
+        else:
+            self.server_options.update(kwargs)
 
     def init_app(self, app, **kwargs):
         if app is not None:
             if not hasattr(app, 'extensions'):
                 app.extensions = {}  # pragma: no cover
             app.extensions['socketio'] = self
-        self.server_options = kwargs
+        self.server_options.update(kwargs)
 
         if 'client_manager' not in self.server_options:
             url = kwargs.pop('message_queue', None)
@@ -144,6 +153,8 @@ class SocketIO(object):
             if url:
                 if url.startswith('redis://'):
                     queue_class = socketio.RedisManager
+                elif url.startswith('zmq'):
+                    queue_class = socketio.ZmqManager
                 else:
                     queue_class = socketio.KombuManager
                 queue = queue_class(url, channel=channel,
@@ -216,7 +227,7 @@ class SocketIO(object):
                 self.server.on(message, _handler, namespace=namespace)
             else:
                 self.handlers.append((message, _handler, namespace))
-            return _handler
+            return handler
         return decorator
 
     def on_error(self, namespace=None):
@@ -314,23 +325,28 @@ class SocketIO(object):
         :param room: Send the message to all the users in the given room. If
                      this parameter is not included, the event is sent to
                      all connected users.
-        :param include_self: ``True`` to include the sender when broadcasting
-                             or addressing a room, or ``False`` to send to
-                             everyone but the sender.
+        :param skip_sid: The session id of a client to ignore when broadcasting
+                         or addressing a room. This is typically set to the
+                         originator of the message, so that everyone except
+                         that client receive the message.
         :param callback: If given, this function will be called to acknowledge
                          that the client has received the message. The
                          arguments that will be passed to the function are
                          those provided by the client. Callback functions can
                          only be used when addressing an individual client.
         """
-        skip_sid = flask.request.sid \
-            if not kwargs.get('include_self', True) else None
-        self.server.emit(event, *args, namespace=kwargs.get('namespace', '/'),
-                         room=kwargs.get('room'), skip_sid=skip_sid,
-                         callback=kwargs.get('callback'))
+        namespace = kwargs.pop('namespace', '/')
+        room = kwargs.pop('room', None)
+        include_self = kwargs.pop('include_self', True)
+        skip_sid = kwargs.pop('skip_sid', None)
+        if not include_self and not skip_sid:
+            skip_sid = flask.request.sid
+        callback = kwargs.pop('callback', None)
+        self.server.emit(event, *args, namespace=namespace, room=room,
+                         skip_sid=skip_sid, callback=callback, **kwargs)
 
     def send(self, data, json=False, namespace=None, room=None,
-             callback=None, include_self=True):
+             callback=None, include_self=True, skip_sid=None, **kwargs):
         """Send a server-generated SocketIO message.
 
         This function sends a simple SocketIO message to one or more connected
@@ -347,22 +363,23 @@ class SocketIO(object):
         :param room: Send the message only to the users in the given room. If
                      this parameter is not included, the message is sent to
                      all connected users.
-        :param include_self: ``True`` to include the sender when broadcasting
-                             or addressing a room, or ``False`` to send to
-                             everyone but the sender.
+        :param skip_sid: The session id of a client to ignore when broadcasting
+                         or addressing a room. This is typically set to the
+                         originator of the message, so that everyone except
+                         that client receive the message.
         :param callback: If given, this function will be called to acknowledge
                          that the client has received the message. The
                          arguments that will be passed to the function are
                          those provided by the client. Callback functions can
                          only be used when addressing an individual client.
         """
-        skip_sid = flask.request.sid if not include_self else None
+        skip_sid = flask.request.sid if not include_self else skip_sid
         if json:
             self.emit('json', data, namespace=namespace, room=room,
-                      skip_sid=skip_sid, callback=callback)
+                      skip_sid=skip_sid, callback=callback, **kwargs)
         else:
             self.emit('message', data, namespace=namespace, room=room,
-                      skip_sid=skip_sid, callback=callback)
+                      skip_sid=skip_sid, callback=callback, **kwargs)
 
     def close_room(self, room, namespace=None):
         """Close a room.
@@ -673,7 +690,7 @@ def send(message, **kwargs):
                          include_self=include_self, callback=callback)
 
 
-def join_room(room):
+def join_room(room, sid=None):
     """Join a room.
 
     This function puts the user in a room, under the current namespace. The
@@ -688,13 +705,15 @@ def join_room(room):
             send(username + ' has entered the room.', room=room)
 
     :param room: The name of the room to join.
+    :param sid: The session id of the client. If not provided, the client is
+                obtained from the request context.
     """
     socketio = flask.current_app.extensions['socketio']
-    socketio.server.enter_room(flask.request.sid, room,
-                               namespace=flask.request.namespace)
+    sid = sid or flask.request.sid
+    socketio.server.enter_room(sid, room, namespace=flask.request.namespace)
 
 
-def leave_room(room):
+def leave_room(room, sid=None):
     """Leave a room.
 
     This function removes the user from a room, under the current namespace.
@@ -709,10 +728,12 @@ def leave_room(room):
             send(username + ' has left the room.', room=room)
 
     :param room: The name of the room to leave.
+    :param sid: The session id of the client. If not provided, the client is
+                obtained from the request context.
     """
     socketio = flask.current_app.extensions['socketio']
-    socketio.server.leave_room(flask.request.sid, room,
-                               namespace=flask.request.namespace)
+    sid = sid or flask.request.sid
+    socketio.server.leave_room(sid, room, namespace=flask.request.namespace)
 
 
 def close_room(room):
@@ -728,16 +749,19 @@ def close_room(room):
     socketio.server.close_room(room, namespace=flask.request.namespace)
 
 
-def rooms():
+def rooms(sid=None):
     """Return a list of the rooms the client is in.
 
     This function returns all the rooms the client has entered, including its
     own room, assigned by the Socket.IO server. This is a function that can
     only be called from a SocketIO event handler.
+
+    :param sid: The session id of the client. If not provided, the client is
+                obtained from the request context.
     """
     socketio = flask.current_app.extensions['socketio']
-    return socketio.server.rooms(flask.request.sid,
-                                 namespace=flask.request.namespace)
+    sid = sid or flask.request.sid
+    return socketio.server.rooms(sid, namespace=flask.request.namespace)
 
 
 def disconnect(silent=False):
